/*
 * Nefit_TopLine_Compact_HRC30CW5.h, boiler definition file.
 *
 * Boiler-specific data for the Nefit TopLine Compact, HRC30/CW5 boiler.
 *
 * (data taken from "Service-instructie Hoog Rendement Gaswandketel Nefit 
 * TopLine Compact HRC, pp91-92", document # 7 746 800 002 (07/2008) NL, 
 * published by Nefit, 2009).
 *
 * Not all numbers are set to a proper level. Some are just made up.
 */

#ifndef __BOILER_DEFINITION_H__
#define __BOILER_DEFINITION_H__

#define BOILER_BRAND  "Nefit"
#define BOILER_TYPE   "TopLine Compact HRC30/CW5"

/* 
 * OT IDs supported by this boiler and toon (not exhaustive, some values 
 * conflict with the list by Schelte Bron: 
 * http://otgw.tclcode.com/matrix.cgi#boilers )  
 */

int supported_OT_IDs[] = {   0,   1,   2,   3,        6,   9,  14,       16,
                          17,  18,       25,  26,  27,  28,  29,        
                          56,  57,                115, 116,           119, 
                         120,                127 };  

/* IDs supported by toon:

int supp_OT_IDs = {   0,    1,   2,   3,   5,   6,   9,  14?, 15,  16,  17,  18,
		      24,  25,  26,  27,  28,  29,  35,  48,  56,  57,  73, 113,
		     114, 115, 116, 117, 118, 119, 120, 121, 122, 123, 127 };  
*/

/* 
 * power as function of relative modulation level. Even though Nefit published
 * different output levels at different boiler temperature ranges, the 
 * differences are small enough to assume a single boiler characteristic.
 *
 * Boilers are fairly linear machines. So a simple linear equation describes
 * power output as function of modulation level remarkably well:
 * 
 * Central heating:
 *
 * Power_CH  = A_P_CH  * rel_mod_level  + B_P_CH [ kW ]
 *
 * and DHW operation:
 *
 * Power_DHW = A_P_DHW * rel_mod_level + B_P_DHW [ kW ]
 *
 * Relative modulation level is reported in OT ID 17.
 */

/* polynomial coefficients for CH operation */

#define MOD_LEVEL_MIN      25  /* minimum absolute modulation level [ % ] */
#define MOD_LEVEL_MAX     100  /* maximum absolute modulation level [ % ] */

#define A_P_CH         0.2257  /* power per relative modulation level 
				  [ kW / % ] */
#define B_P_CH         6.7333  /* power at minimum absolute modulation level
				  [ kW ] */

/* polynomial coefficients for DHW operation */

#define A_P_DHW        0.2680  /* power per relative modulation level 
				  [ kW / % ] */
#define B_P_DHW        6.6000  /* power at minimum absolute modulation level
				  [ kW ] */

/* Thermal properties of the boiler (taken from measurements and OT logs) */

#define M_C_P_DHW         384  /* heat capacity of the boiler in DHW 
				  operation. [J/K] */
   
/* Timing properties */

#define TRANSPORT_DELAY_CH  30 /* dead time between flame on and output temp 
				  increase at boiler output [s] */
#define TAU_DHW           5000 /* characteristic time for heat loss in DHW
				  circuit of the boiler [s] */
  
/* modulation level (percentage of maximum), the same for HRC30 and HRC25 */

double Nefit_HRCxxCWx_mod_perc[16] = 
  { 25, 30, 35, 40, 45, 50, 55, 60, 
    65, 70, 75, 80, 85, 90, 95, 100 }; 

double Nefit_HRCxxCWx_mod_perc_min_max[2] = { 25, 100 };

/* HRC30/CW5 */

/* power output [kW] at 40/30, 80/60, 50/30 deg C, and DHW mode */
double Nefit_HRC30CW5_mod_power_40_30[16] = 
  {  6.6,  8.1,  9.6, 11.1, 12.6, 14.1, 15.6, 17.1, 
    18.6, 20.1, 21.6, 23.1, 24.6, 26.1, 27.6, 29.2 };

double Nefit_HRC30CW5_mod_power_80_60[2] = { 6.4, 28.5 }; 
double Nefit_HRC30CW5_mod_power_50_30[2] = { 7.2, 30.2 };
double Nefit_HRC30CW5_mod_power_DHW[2]   = { 6.6, 33.4 };

/*
 * CH_heat capacity is estimated from the weight of the boiler (42kg, 
 * manufacturer's info), the fraction of that  weight that's being heated (65%, 
 * rough estimate), and the heat capacity c_p of that heated material (aluminium 
 * in this case, mostly the burner and heat exchanger, 0.91kJ/kgK). 
 */

double CH_heat_capacity = 24.8; /* kJ/K */

/* 
 * DHW heat capacity is mostly the heat exchanger: a SWEP E6T, 20 plates.
 * weight: 1.14kg, stainless steel AISI316, c_p = 0.502 kJ/kgK. 
 */

double DHW_heat_capacity = 0.57; /* kJ/K */

/* HRC25/CW4 */
 
/* power output [kW] at 40/30, 80/60, 50/30 deg C, and DHW mode */

double Nefit_HRC25CW4_mod_power[16] = 
  {  5.0,  7.2,  8.4,  9.6, 10.8, 12.0, 13.2, 14.4, 
    15.6, 16.8, 18.0, 19.2, 20.4, 21.6, 22.8, 23.5 };

double Nefit_HRC25CW4_mod_power_80_60[2] = { 4.8, 22.9 }; 
double Nefit_HRC25CW4_mod_power_50_30[2] = { 5.5, 24.4 };
double Nefit_HRC25CW4_mod_power_DHW[2]   = { 5.0, 28.4 };



#endif
