/*
 * boiler_emu.c
 *
 * Code to emulate a domestic CH boiler using Schelte Bron's OTGW.
 * (http://otgw.tclcode.com)
 *
 * The OTGW needs to have the interface.hex firmware installed on it, 
 * version 1.1. 
 * This is not checked. It's up to the user to have the firmware
 * arranged properly.
 *
 * 20160527, marcelr
 * first draft.
 */ 

#include <stdio.h>
#include <termios.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/select.h>
#include <fcntl.h>
#include <stdlib.h>
#include <pthread.h>
#include <time.h>

#include "OT_ids.h"
#include "boilers.h"

#define DEBUG                           0

/* serial port settings */

/*
 * Add the following to /etc/udev/rules.d/10-local.rules for the link to 
 * OTGW_PORT to work (check actual device ATTRS from dmesg output, this is for
 * my FTDI USB-to-serial interface):
 *
 * # USB to serial, control port of OTGW:
 * ACTION=="add",                               \
 * SUBSYSTEM=="usb",                            \
 * ATTRS{idVendor}=="0403",			\
 * ATTRS{idProduct}=="6001",                    \
 * ATTRS{serial}=="FTGHPZFA" SYMLINK+="OTGW"
 *
 */

#define OTGW_PORT             "/dev/OTGW"
#define N_TRIES                        10 /* # tries before read is abandoned */
#define SERIAL_WAIT_SEC                 0
#define SERIAL_WAIT_USEC            20000 /* 0.02 s */

/* OpenTherm command types */

/* from master to slave (thermostat to boiler) */

#define M_S_READ_DATA                   0
#define M_S_WRITE_DATA                  1
#define M_S_INVALID_DATA                2
#define M_S_RESERVED                    3

/* from slave to master (boiler to thermostat) */

#define S_M_READ_ACK                    4
#define S_M_WRITE_ACK                   5
#define S_M_DATA_INVALID                6
#define S_M_UNKNOWN_DATAID              7

#define BUF_LEN                        80 /* should fit any interface message */

/* function prototypes */

/* Hardware IO */

int            open_serial_port( char* );
void           init_serial_port( int, struct termios*, struct termios* );
void           close_serial_port( int, struct termios* );

/* general helper functions */

int            check_root( void );
unsigned int   check_parity( unsigned int );
unsigned int   read_until_valid( int );
int            parse_and_reply( int, unsigned int, unsigned short* );
double         OT_float_to_double( unsigned short );
unsigned short double_to_OT_float( double );

/* Threads */

void*          heating_system( void* );
void*          boiler_IO( void* );

/* Models for the heating system */

double         boiler_model( unsigned short * );
double         CH_model( unsigned short *, double );

/* Timing stuff */

static inline double get_timestamp( struct timeval* );
void set_normalized_timespec( struct timespec *, time_t, long long );
struct timespec timespec_sub( struct timespec, struct timespec );
struct timespec timespec_add( struct timespec, struct timespec );

#define NSEC_PER_SEC      1000000000L
#define ACQ_INTERVAL_SEC  1
#define ACQ_INTERVAL_NSEC 0




/* Global stuff to be exchanged between threads */

pthread_mutex_t ID_mutex; 
unsigned short* ID_data;


int main( void )
{
  pthread_t       thread_id[2];
  void*           status;
  int             i;
  /* 
   * ID_data is an array with 128 entries, holding data prepared by the boiler
   * model. It is filled/read asynchronously, in both threads.
   */

  ID_data = malloc( sizeof( unsigned short ) * 128 );
 
  pthread_mutex_init( &ID_mutex, NULL );             

  if ( !pthread_create( &thread_id[0], NULL, &heating_system, 
			(void *)ID_data ) )
    printf( "created heating_model thread\n" );

  if ( !pthread_create( &thread_id[1], NULL, &boiler_IO, 
			(void *)ID_data ) )
    printf( "created boiler_IO thread\n" );

  for ( i = 0; i < 2; i++ )
    {
      pthread_join( thread_id[i], &status );
    }    

  pthread_mutex_destroy( &ID_mutex );
  
  free( ID_data );
  
  return 0;
}



void* boiler_IO(void* arg )
/*
 * Thread for boiler communication.
 *
 * Talks to the OTGW independent of the boiler behaviour: It replies to requests
 * from the thermostat, and takes its data from (unsigned short)ID_data[];
 *
 * The OTGW is the slave in the communication, so we just need to wait for
 * the thermostat (the master) to start talking. Only then it's time to reply
 * (sensibly, with a little luck ;-) ).
 */
{
  int             port;
  struct termios  oldtio, newtio;
  unsigned int    msg;

  unsigned short* data = (unsigned short*)arg;

 /* prepare hardware */

  check_root();

  port = open_serial_port( OTGW_PORT );
  init_serial_port( port, &oldtio, &newtio );
  
  /* 
   * Read whatever appears on the OpenTherm lines of the gateway, and reply
   * accordingly.
   */

  while ( 1 ) /* kill with ctrl+c for now */
    {
      msg = read_until_valid( port );

      pthread_mutex_lock( &ID_mutex );

      parse_and_reply( port, msg, ID_data );

      pthread_mutex_unlock( &ID_mutex );
    }

  close_serial_port( port, &oldtio );

}



void* heating_system( void* arg )
/*
 * Thread for heating installation behaviour.
 * Main routine for the heating model.
 *
 * Fills (unsigned short)ID_data[] with relevant data, and takes ID0 and ID1 
 * input from it.
 * 
 */
{
  unsigned short* data = (unsigned short*)arg;
  double noise;
  double T_out, T_in;

  /* control loop timing */

  struct timespec *delta;
  struct timespec acq_clock;

  struct timeval *t1;

  double t   = 0;
  double t_o = 0;
  double dt;


  printf("Model for %s boiler, type %s\n\n", BOILER_BRAND, BOILER_TYPE );

  /* Set up model timer: */

  t1    = malloc(sizeof(struct timeval));
  delta = malloc(sizeof(struct timespec));

  /* get first reading for t_o */

  clock_gettime( CLOCK_REALTIME, &acq_clock );
  t_o = (double)(acq_clock.tv_sec) + 1e-9 * (double)(acq_clock.tv_nsec);

  /* set sampling interval */

  delta->tv_sec  = ACQ_INTERVAL_SEC;
  delta->tv_nsec = ACQ_INTERVAL_NSEC;

  while ( 1 )
    {
      /* update time request and wait until next sample is due */
      
      acq_clock = timespec_add( acq_clock, *delta );
      clock_nanosleep( CLOCK_REALTIME, TIMER_ABSTIME, &acq_clock, NULL );
            
      /* calculate timestamp and actual sampling interval */
      
      t   = get_timestamp( t1 );
      dt  = t - t_o;
      t_o = t;

      /*
       * Do all model calculations here.
       */

      noise = (double)rand() / (double)RAND_MAX;
      noise -= 0.5;

      T_out = boiler_model( ID_data );
      T_in  = CH_model( ID_data, T_out );

      /*
       * Calculations finished, now write to data buffer.
       */

      pthread_mutex_lock( &ID_mutex );

      ID_data[ID_BOILER_WATER_TEMP]  = double_to_OT_float( T_out );
      ID_data[ID_RETURN_WATER_TEMP]  = double_to_OT_float( T_in );
      ID_data[ID_BOILER_WATER_PRESS] = double_to_OT_float( 1.97 );

      pthread_mutex_unlock( &ID_mutex );
    }

  free( delta );
  free( t1 );

}



double boiler_model( unsigned short* ID_data )
/*
 * Model for a modulating boiler. Takes inputs from OT ID_data and returns 
 * boiler output temperature.
 *
 * placeholder code, nothing serious yet.
 */
{
  double power;
  double T_set;
  double T_in, T_out, T_ambient;
  double mod_level;

  T_set     = OT_float_to_double( ID_data[ID_BOILER_CTRL_SETPOINT] );
  T_in      = OT_float_to_double( ID_data[ID_RETURN_WATER_TEMP] );
  T_out     = OT_float_to_double( ID_data[ID_BOILER_WATER_TEMP] );
  T_ambient = OT_float_to_double( ID_data[ID_OUTSIDE_TEMP] );

  T_out = 23.6;
  return T_out;
}



double CH_model( unsigned short* ID_data, double T_out ) 
/*
 * Model for central heating installation (excluding the boiler).
 *
 * Takes boiler output temperature (T_out) as input and returns return 
 * temperature (T_in).
 *
 * placeholder code, nothing serious yet.
 */
{
  double T_in, T_ambient;

  T_ambient = OT_float_to_double( ID_data[ID_OUTSIDE_TEMP] );

  T_in = 16.8;
  return T_in;

}



int parse_and_reply( int port, unsigned int msg, unsigned short* ID_data )
{
  /*
   * Parse data as read from OTGW, and display on screen. 
   *
   * formulate a reply and send it (not yet working, placeholder code for now).
   */
  unsigned int   reply_msg;
  int            msg_flag;
  int            rep_flag;
  int            i;
  unsigned char  OT_id;
  unsigned short OT_payload;
  char           reply[11];
  double         dval;
  unsigned char  bitval[16];
  unsigned short usval;
  char           cval[2];
  unsigned char  ucval[2];

  /* decompose message */

  msg_flag   = (int)( ( msg & 0x70000000 ) >> 28 );
  OT_id      = (unsigned char)( ( msg & 0x00FF0000 ) >> 16 );
  OT_payload = (unsigned short)( msg & 0x0000FFFF );
  
  printf("msg_flag = %d, ID %3d, data = 0x%04x ", 
	 msg_flag, OT_id, OT_payload );

  /* set message flag for reply message */

  switch ( msg_flag )
    {
    case M_S_READ_DATA:
      rep_flag = S_M_READ_ACK;
      break;
    case M_S_WRITE_DATA:
      rep_flag = S_M_WRITE_ACK;
      ID_data[OT_id] = OT_payload;
     break;
    case S_M_READ_ACK:
    case S_M_WRITE_ACK:
      rep_flag = -1;
      break;
    default:
      return -1;
      break;
    }

  /* 
   * Assess payload.
   *
   * OT IDs as implemented in Eneco's toon have been set up so
   * far, others may be added according to specific needs.
   */

  switch (OT_id )
    {
      /* bit fields */

    case 0: /* status byte: high byte: master, low byte: slave                */
    case 2: /* high byte: master configuration, low byte: master member ID    */
    case 3: /* high byte: slave configuration, low byte: slave member ID      */
    case 5: /* high byte: app-specific fault flags, low byte: OEM fault code  */
    case 6: /* high byte: remote par enable flag, low byte: remote par r/w 
	       flag */

      bitval[ 0] =   OT_payload & 0x0001;
      bitval[ 1] = ( OT_payload & 0x0002 ) >> 1;
      bitval[ 2] = ( OT_payload & 0x0004 ) >> 2;
      bitval[ 3] = ( OT_payload & 0x0008 ) >> 3;
      bitval[ 4] = ( OT_payload & 0x0010 ) >> 4;
      bitval[ 5] = ( OT_payload & 0x0020 ) >> 5;
      bitval[ 6] = ( OT_payload & 0x0040 ) >> 6;
      bitval[ 7] = ( OT_payload & 0x0080 ) >> 7;
      bitval[ 8] = ( OT_payload & 0x0100 ) >> 8;
      bitval[ 9] = ( OT_payload & 0x0200 ) >> 9;
      bitval[10] = ( OT_payload & 0x0400 ) >> 10;
      bitval[11] = ( OT_payload & 0x0800 ) >> 11;
      bitval[12] = ( OT_payload & 0x1000 ) >> 12;
      bitval[13] = ( OT_payload & 0x2000 ) >> 13;
      bitval[14] = ( OT_payload & 0x4000 ) >> 14;
      bitval[15] = ( OT_payload & 0x8000 ) >> 15;

      printf("data = " );
      for ( i = 15; i >= 0; i-- )
	{
	  printf("%d", bitval[i] );
	  if ( i == 8 )
	    printf(" ");
	}
      break;

    case 1:  /* boiler control setpoint [deg C]       */
    case 9:  /* remote override room setpoint [deg C] */
    case 14: /* maximum modulation level setting [%]  */ 
    case 15: /* maximum boiler power [kW]             */
    case 16: /* room temperature setpoint [deg C]     */
    case 17: /* relative modulation level [0-100%]    */
    case 18: /* boiler water pressure [bar]           */
    case 24: /* room temperature [deg C]              */
    case 25: /* boiler water temperature [deg C]      */
    case 26: /* DHW temperature [deg C]               */
    case 27: /* Outside temperature [deg C]           */
    case 28: /* return water temperature [deg C]      */
    case 29: /* solar storage temperature [deg C]     */
    case 56: /* max DHW setpoint [deg C]              */
    case 57: /* max CH water setpoint [deg C]         */
      
      dval = OT_float_to_double( OT_payload );
      printf( "data = %11.2f", dval );
      break;

    /* two signed characters */

    case 35: /* boiler fan speed and setpoint                           */
    case 48: /* high byte: DHW temp high bound, low: DHW temp low bound */
    
      cval[0] = (char)(OT_payload & 0x00ff);
      cval[1] = (char)((OT_payload & 0xff00) >> 8 );
      printf("data = %8d %d", cval[1], cval[0] );
      break;

      /* unsigned short */

    case 73:  /* OEM-specific code for ventilation / heat recovery 
		 (unknown data format, stored as unsigned short for now */
    case 113: /* number of unsuccessful burner starts     */
    case 114: /* number of times flame signal was too low */
    case 115: /* OEM diagnostic code                      */
    case 116: /* number of burner starts                  */
    case 117: /* number of CH pump starts                 */
    case 118: /* number of DHW pump/valve starts          */
    case 119: /* number of burner starts for DHW          */
    case 120: /* burner operation hours                   */
    case 121: /* CH pump hours                            */
    case 122: /* DHW pump/valve hours                     */
    case 123: /* DHW burner hours                         */

      usval = OT_payload;
      printf( "data = %8d", usval );
      break;

      /* two unsigned characters */
      
    case 127: /*slave number and type */

      ucval[0] = (unsigned char)(OT_payload & 0x00ff);
      ucval[1] = (unsigned char)((OT_payload & 0xff00) >> 8 );
      printf("data = %8d %d", ucval[1], ucval[0] );
      break;
     
    default:
      break;
    }

  printf("\n");

  /* formulate a reply when necessary */

  if ( rep_flag > 0 )
    {
      reply_msg = 0;

      /* fill in data from boiler_model as required 
       *
       * placeholder code for now.
       */

      switch ( OT_id )
        {
        case ID_BOILER_WATER_PRESS:
        case ID_BOILER_WATER_TEMP:
	case ID_RETURN_WATER_TEMP:
	  OT_payload = ID_data[OT_id];
	  break;
	  
	default:
	  break;
	}

      /* compose reply string and write to OTGW */
      
      reply_msg = ((unsigned int) OT_payload) | (((unsigned int)OT_id) << 16) |
	(((unsigned int) rep_flag) << 28 );  
      
      if ( check_parity( reply_msg ) )
        {
	  reply_msg |= 0x80000000;
	}
      
      sprintf( reply, "%08x\r\n", reply_msg );
      if ( DEBUG )
	printf("reply = %s", reply ); 
      
      write( port, reply, 10 );
    }
  return 0;
}



unsigned short double_to_OT_float( double val )
{
  /* 
   * convert double to unsigned short (f8.8 float, actually) according to the
   * OT protocol. 
   */

  short hlp;
  unsigned short ret;

  val *= 256.0;
  hlp = (short)val;
  memcpy( &ret, &hlp, sizeof( short ) );

  return ret;
}



double OT_float_to_double( unsigned short val )
{
  /* 
   * convert unsigned short (f8.8 float really) to double according to the 
   * OT protocol. 
   */
  short h, l;
  h = (short)( val >> 8 );
  h = h << 8;
  l = (short)( val & 0x00ff );

  return ( (double) ( h | l ) ) / 256.0;
}



unsigned int read_until_valid( int port )
{
  /*
   * Read data from OTGW from serial port. 
   * Strip leading character ("B" or "T"), trailing "\r\n" and return actual 
   * data.  
   */
  char msg_type;
  char buf[BUF_LEN];
  int n = -1;
  int i, k;
  unsigned int msg;

  while ( n == -1 )
    {
      /* clear buffer */
      
      memset( buf, 0, BUF_LEN );
      
      /* and fill it again */
      
      n = read_from_port( port, buf, BUF_LEN );
      if ( DEBUG )
	printf("%d bytes read:\n%s\n", n, buf );
      if ( n > 0 )
	{
	  if ( DEBUG )
	    {
	      for ( i = 0; i < n; i++ )
		{
		  printf("%02x ", buf[i] );
		}
	      printf("\n");
	    }
	  
	  /* read buffer contents */
	  
	  k =  sscanf( buf, "%c%08x\r\n", &msg_type, &msg );
	  if ( k < 2 | n < 11 )
	    {
	      printf("line has no valid data\n");
	      n   = -1;
	      msg = 0;
	    }
	  else
	    {
	      if ( DEBUG )
		printf("type = %c, msg = 0x%08x\n", msg_type, msg );
	      if ( check_parity( msg ) )
		{
		  printf("parity error\n");
		  n   = -1;
		  msg = 0;
		}
	      else
		return msg;
	    }
	}
    }      
}



int open_serial_port( char* device )
/* 
 * open_serial_port opens serial port with device special file device,
 * e.g., /dev/ttyUSB0, and on success returns a valid file descriptor for this
 * port.
 */
{
  int fd;
  /* 
   * Open serial device for reading and writing and not as controlling tty
   * because we don't want to get killed if linenoise sends CTRL-C.
   */
  fd = open( device, O_RDWR | O_NOCTTY );

  if ( fd < 0 ) 
    { 
      perror( device ); 
      exit( EXIT_FAILURE ); 
    } 
  else 
    {
      return fd;
    }
}



void init_serial_port( int fd, struct termios* oldtio, struct termios* newtio )
/*
 * init_serial_port initializes serial port with file descriptor fd for use 
 * with an OTGW.
 * The original port settings are stored in termios struct oldtio, the new
 * port configuration is stored in newtio.
 *
 * Configuration parameters have been copied from a working OTGW, sw version 
 * interface.hex, v1.1.
 */

{
  tcgetattr( fd, oldtio );            /* save current serial port settings */

  memset( newtio, 0x0, sizeof( *newtio ) ); /* clear tty struct */

  /*
   * Initialize input modes.
   */

  newtio->c_iflag = 0;

  /*
   * Initialize output modes, no idea why these are set, you don't need them
   * in raw transfers.
   *
   * NLDLY  Newline delay mask. Values are NL0 and NL1.
   * CRDLY  Carriage return delay mask. Values are CR0, CR1, CR2, or CR3.
   * TABDLY Horizontal  tab  delay  mask.  Values are TAB0, TAB1, TAB2, TAB3
   *        (or XTABS).  A value of TAB3, that is, XTABS,  expands  tabs  to
   *        spaces   (with   tab  stops  every  eight  columns).
   * BSDLY  Backspace delay mask. Values are BS0 or BS1. (Has  never  been
   *        implemented.) 
   * VTDLY  Vertical tab delay mask. Values are VT0 or VT1.
   * FFDLY  Form feed delay  mask. Values are FF0 or FF1.
   */

  newtio->c_oflag = NL0 | CR0 | TAB0 | BS0 | VT0 | FF0;  

  /*
   * Initialize control modes.
   *
   * "standard" port settings: 
   * B9600   : Set baud rate. 9600 for Schelte's OTGW.
   * CS8     : 8n1 (8bit,no parity,1 stopbit)
   * CLOCAL  : local connection, no modem control
   * CREAD   : enable receiving characters
   */

  newtio->c_cflag = B9600 | CS8 | CLOCAL | CREAD;

  /*
   * Initialize local modes.
   */

  newtio->c_lflag = 0;

  /* 
   * Initialize control characters 
   *
   * VTIME    : Timeout in deciseconds for non-canonical read.
   * VMIN     : Minimum number of characters for non-canonical read.
   */

  newtio->c_cc[VTIME]  = 0;        /* inter-character timer used          */
  newtio->c_cc[VMIN]   = 1;        /* blocking read until 1 char arrives. */

  /* 
   * Activate settings 
   */ 

  tcflush( fd, TCIFLUSH );
  tcsetattr( fd, TCSANOW, newtio );
}



void close_serial_port( int fd, struct termios* tio )
/*
 * close_serial_port() closes serial port with file descriptor fd and restores
 * the port settings to the ones stored in tio.
 */
{
  tcsetattr( fd, TCSANOW, tio );
  close( fd );
}



int check_root( void )
/* 
 * test if the program runs with suid root. If not, exit. 
 */
{
  if ( getuid() != 0 )
    {
      printf( "This program must be run by root (user id 0).\n" );
      printf( "Using sudo is fine too ...\n " );
      exit(-1);
    } 
  else 
    {
      return 0;
    }
}



int read_from_port( int port, unsigned char* buf, int n )
{
  /*
   * reads n bytes from port port, one by one, and stores them in buf.
   * It is up to the programmer to make the buffer sufficiently large.
   *
   * returns the number of bytes read, or a negative number in case of error.
   */
  int ret;
  int cnt;
  struct timeval timeout;
  fd_set rdset;
  int maxfd;
  int i = 0;
  int j = 0;
  ret = -1;
  cnt = 0;
  
  /* keep reading until we've got what we came for */

  while ( j < n )
    {

      /* keep trying until the port is unblocked, or N_TRIES is exceeded */

      while ( (cnt < N_TRIES) & (ret <= 0 ) )
	{
	  timeout.tv_sec  = SERIAL_WAIT_SEC;
	  timeout.tv_usec = SERIAL_WAIT_USEC;
	  
	  FD_ZERO( &rdset );
	  FD_SET( port, &rdset );
	  maxfd = port + 1;
	  
	  ret = select( maxfd, &rdset, NULL, NULL, &timeout );
	  cnt++;
	}
      switch ( ret ) 
	{
	case 0:
	  /* timeout, can happen, nothing to be worried about */
	  //perror("read_from_port: ");
	  return -1;
	  break;
	default:
	  
	  /* read bytes at serial port */
	  
	  ret < 0 ? ( perror("read_from_port: ") ) : 
	  ( i = read( port, buf + j, 1 ) );


	  if ( buf[j] == 0xa )
	    {
	      if ( DEBUG ) 
		printf("EOL\n");
	      j += i;
	      return j;
	    }
	  j += i;
	  
	  if ( DEBUG )
	    printf( "read_from_port: %2d bytes read, %2d of %2d in total\n", 
		    i, j, n );
	  
	  break;
	}
    }

  if ( ret < 0 )
    return -2;
  else
    return j;
}



unsigned int check_parity( unsigned int x )
{
  /*
   * copied from: 
   * https://graphics.stanford.edu/~seander/bithacks.html#ParityMultiply
   * retrieved 20160528. 
   */
  x ^= x >> 1;
  x ^= x >> 2;
  x = (x & 0x11111111U) * 0x11111111U;
  return (x >> 28) & 1;
}


int bit_count( unsigned int v )
{
  /*
   * Not really needed yet. But I found it interesting, and keep it here, just in 
   * case ...
   *
   * copied from: 
   * https://graphics.stanford.edu/~seander/bithacks.html#ParityMultiply
   * retrieved 20160528. 
   *
   * Counting bits set in 14, 24, or 32-bit words using 64-bit instructions
   */
  //unsigned int v; /* count the number of bits set in v */
  unsigned int c; /* c accumulates the total bits set in v */
  
  
  /* option 1, for at most 14-bit values in v: */
  /*
    c = (v * 0x200040008001ULL & 0x111111111111111ULL) % 0xf;
  */
  /* option 2, for at most 24-bit values in v: */
  
  /*
    c =  ((v & 0xfff) * 0x1001001001001ULL & 0x84210842108421ULL) % 0x1f;
    c += (((v & 0xfff000) >> 12) * 0x1001001001001ULL & 0x84210842108421ULL) 
    % 0x1f;
  */

  /* option 3, for at most 32-bit values in v: */
  c =  ((v & 0xfff) * 0x1001001001001ULL & 0x84210842108421ULL) % 0x1f;
  c += (((v & 0xfff000) >> 12) * 0x1001001001001ULL & 0x84210842108421ULL) % 
    0x1f;
  c += ((v >> 24) * 0x1001001001001ULL & 0x84210842108421ULL) % 0x1f;
  
  /*
   * This method requires a 64-bit CPU with fast modulus division to be 
   * efficient.
   * The first option takes only 3 operations; the second option takes 10; and 
   * the third option takes 15.
   *
   * Rich Schroeppel originally created a 9-bit version, similiar to option 1; 
   * see the Programming Hacks section of Beeler, M., Gosper, R. W., and 
   * Schroeppel, R. HAKMEM. MIT AI Memo 239, Feb. 29, 1972. His method was the 
   * inspiration for the variants above, devised by Sean Anderson. 
   * Randal E. Bryant offered a couple bug fixes on May 3, 2005. Bruce Dawson 
   * tweaked what had been a 12-bit version and made it suitable for 14 bits 
   * using the same number of operations on Feburary 1, 2007. 
   */
  return (int)c;
}



static inline double get_timestamp( struct timeval *t ) 
/* 
 * calculate timestamp
 * struct t needs to be malloc'd prior to calling this function.
 * double precision needed for sub-millisecond timing accuracy.
 */
{
  gettimeofday( t, NULL ); 
  return (double)( t->tv_sec ) + 1e-6 * (double)( t->tv_usec );
}



/* timespec stuff to use ns time resolution */

void set_normalized_timespec( struct timespec *ts, time_t sec, long long nsec )
{
  while (nsec >= NSEC_PER_SEC)
  {
    /*
    * The following asm() prevents the compiler from
    * optimising this loop into a modulo operation. See
    * also __iter_div_u64_rem() in include/linux/time.h
    */

    asm("" : "+rm"(nsec));
    nsec -= NSEC_PER_SEC;
    ++sec;
  }

  while (nsec < 0)
  {
    asm("" : "+rm"(nsec));
    nsec += NSEC_PER_SEC;
    --sec;
  }
  ts->tv_sec = sec;
  ts->tv_nsec = nsec;
}



struct timespec timespec_sub(struct timespec lhs,struct timespec rhs)
{
  struct timespec ts_delta;
  /* subtract normalized time and return a struct timespec */

  set_normalized_timespec( &ts_delta, lhs.tv_sec - rhs.tv_sec,
			   lhs.tv_nsec - rhs.tv_nsec );
  return ts_delta;
}



struct timespec timespec_add(struct timespec lhs, struct timespec rhs)
{
  /* add normalized time and return a struct timespec */

  struct timespec ts_delta; 
  set_normalized_timespec( &ts_delta, lhs.tv_sec + rhs.tv_sec, 
			   lhs.tv_nsec + rhs.tv_nsec );
  return ts_delta;
}
